@@ -54,6 +54,12 @@
 #include "url.h"
 #include "xchatc.h"
 
+#ifdef USE_DCC64
+#define BIG_STR_TO_INT(x) strtoull(x,NULL,10)
+#else
+#warning Not able to use 64-bit DCC
+#define BIG_STR_TO_INT(x) strtoul(x,NULL,10)
+#endif
 
 static char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
 
@@ -77,7 +83,15 @@ static gboolean dcc_read_ack (GIOChannel *source, GIOCondition condition, struct
 
 static int new_id()
 {
-	static int id = 1;
+	static int id = 0;
+	if (id == 0)
+	{
+		/* start the first ID at a random number for pseudo security */
+		srand (time (0));
+		/* 1 - 255 */
+		id = 1 + (int) ((float)255 * rand()/(RAND_MAX+1.0));
+		/* ignore overflows, since it can go to 2 billion */
+	}
 	return id++;
 }
 
@@ -113,7 +127,7 @@ dcc_calc_cps (struct DCC *dcc)
 	double timediff, startdiff;
 	int glob_throttle_bit, wasthrottled;
 	int *cpssum, glob_limit;
-	unsigned int pos, posdiff;
+	DCC_SIZE pos, posdiff;
 
 	g_get_current_time (&now);
 
@@ -455,6 +469,8 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 	gsize utf_len;
 
 	len = strlen (line);
+	if (dcc->serv->using_cp1255)
+		len++;	/* include the NUL terminator */
 
 	if (dcc->serv->encoding == NULL)     /* system */
 		utf = g_locale_to_utf8 (line, len, NULL, &utf_len, NULL);
@@ -467,6 +483,9 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		len = utf_len;
 	}
 
+	if (dcc->serv->using_cp1255 && len > 0)
+		len--;
+
 	/* we really need valid UTF-8 now */
 	conv = text_validate (&line, &len);
 
@@ -506,7 +525,7 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		return 0;
 	}
 
-	url_check (line);
+	url_check_line (line, len);
 
 	if (line[0] == 1 && !strncasecmp (line + 1, "ACTION", 6))
 	{
@@ -516,7 +535,7 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		inbound_action (sess, dcc->serv->nick, dcc->nick, line + 8, FALSE);
 	} else
 	{
-		inbound_privmsg (dcc->serv, dcc->nick, "", line);
+		inbound_privmsg (dcc->serv, dcc->nick, "", line, FALSE);
 	}
 	if (utf)
 		g_free (utf);
@@ -543,14 +562,14 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		}
 
 		if (!dcc->iotag)
-			dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read, dcc);
+			dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 
 		len = recv (dcc->sok, lbuf, sizeof (lbuf) - 2, 0);
 		if (len < 1)
 		{
 			if (len < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
 					return TRUE;
 			}
 			sprintf (tbuf, "%d", dcc->port);
@@ -573,7 +592,7 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 
 				if (prefs.stripcolor)
 				{
-					temp = strip_color (dcc->dccchat->linebuf);
+					temp = strip_color (dcc->dccchat->linebuf, -1, 1, 1);
 					dead = dcc_chat_line (dcc, temp, tbuf);
 					free (temp);
 				} else
@@ -612,13 +631,21 @@ dcc_calc_average_cps (struct DCC *dcc)
 		dcc->cps = (dcc->pos - dcc->resumable) / sec;
 }
 
+static void
+dcc_send_ack (struct DCC *dcc)
+{
+	/* send in 32-bit big endian */
+	guint32 pos = htonl (dcc->pos & 0xffffffff);
+	send (dcc->sok, (char *) &pos, 4, 0);
+}
+
 static gboolean
 dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	char *old;
 	char buf[4096];
-	guint32 pos;
 	int n;
+	gboolean need_ack = FALSE;
 
 	if (dcc->fp == -1)
 	{
@@ -670,6 +697,9 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	{
 		if (dcc->throttled)
 		{
+			if (need_ack)
+				dcc_send_ack (dcc);
+
 			fe_input_remove (dcc->iotag);
 			dcc->iotag = 0;
 			return FALSE;
@@ -683,12 +713,19 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		{
 			if (n < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
+				{
+					if (need_ack)
+						dcc_send_ack (dcc);
 					return TRUE;
+				}
 			}
 			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
 							 dcc->destfile, dcc->nick,
 							 errorstring ((n < 0) ? sock_error () : 0), 0);
+			/* send ack here? but the socket is dead */
+			/*if (need_ack)
+				dcc_send_ack (dcc);*/
 			dcc_close (dcc, STAT_FAILED, FALSE);
 			return TRUE;
 		}
@@ -697,17 +734,19 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		{
 			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
 							 dcc->destfile, dcc->nick, errorstring (errno), 0);
+			if (need_ack)
+				dcc_send_ack (dcc);
 			dcc_close (dcc, STAT_FAILED, FALSE);
 			return TRUE;
 		}
-		dcc->pos += n;
-		pos = htonl (dcc->pos);
-		send (dcc->sok, (char *) &pos, 4, 0);
 
 		dcc->lasttime = time (0);
+		dcc->pos += n;
+		need_ack = TRUE;	/* send ack when we're done recv()ing */
 
 		if (dcc->pos >= dcc->size)
 		{
+			dcc_send_ack (dcc);
 			dcc_calc_average_cps (dcc);
 			sprintf (buf, "%d", dcc->cps);
 			dcc_close (dcc, STAT_DONE, FALSE);
@@ -718,6 +757,20 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	}
 }
 
+static void
+dcc_open_query (session *sess, char *nick)
+{
+	char *cmd;
+
+	if (prefs.autodialog)
+	{
+		cmd = malloc (8 + strlen (nick));
+		sprintf (cmd, "query %s", nick);
+		handle_command (sess, cmd, FALSE);
+		free (cmd);
+	}
+}
+
 static gboolean
 dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
@@ -793,7 +846,9 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 		EMIT_SIGNAL (XP_TE_DCCCONSEND, dcc->serv->front_session,
 						 dcc->nick, host, dcc->file, NULL, 0);
 		break;
-	case TYPE_CHATRECV:
+	case TYPE_CHATSEND:	/* pchat */
+		dcc_open_query (dcc->serv->server_session, dcc->nick);
+	case TYPE_CHATRECV:	/* normal chat */
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
@@ -832,8 +887,8 @@ dcc_connect (struct DCC *dcc)
 		/* possible problems with filenames containing spaces? */
 		if (dcc->type == TYPE_RECV)
 			snprintf (tbuf, sizeof (tbuf), strchr (dcc->file, ' ') ?
-					"DCC SEND \"%s\" %lu %d %u %d" :
-					"DCC SEND %s %lu %d %u %d", dcc->file,
+					"DCC SEND \"%s\" %lu %d %"DCC_SFMT" %d" :
+					"DCC SEND %s %lu %d %"DCC_SFMT" %d", dcc->file,
 					dcc->addr, dcc->port, dcc->size, dcc->pasvid);
 		else
 			snprintf (tbuf, sizeof (tbuf), "DCC CHAT chat %lu %d %d",
@@ -937,7 +992,7 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	{
 		if (len < 0)
 		{
-			if (would_block_again ())
+			if (would_block ())
 				return TRUE;
 		}
 		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
@@ -951,26 +1006,38 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	recv (sok, (char *) &ack, 4, 0);
 	dcc->ack = ntohl (ack);
 
-	/* fix for BitchX */
-	if (dcc->ack < dcc->resumable)
-		dcc->ackoffset = TRUE;
-	if (dcc->ackoffset)
-		dcc->ack += dcc->resumable;
+	/* this could mess up when xfering >32bit files */
+	if (dcc->size <= 0xffffffff)
+	{
+		/* fix for BitchX */
+		if (dcc->ack < dcc->resumable)
+			dcc->ackoffset = TRUE;
+		if (dcc->ackoffset)
+			dcc->ack += dcc->resumable;
+	}
 
 	/* DCC complete check */
-	if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
+	if (dcc->pos >= dcc->size && dcc->ack >= (dcc->size & 0xffffffff))
 	{
+		dcc->ack = dcc->size;	/* force 100% ack for >4 GB */
 		dcc_calc_average_cps (dcc);
 		dcc_close (dcc, STAT_DONE, FALSE);
 		sprintf (buf, "%d", dcc->cps);
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
 	}
-	else if ((!dcc->fastsend) && (dcc->ack >= dcc->pos))
+	else if ((!dcc->fastsend) && (dcc->ack >= (dcc->pos & 0xffffffff)))
 	{
 		dcc_send_data (NULL, 0, (gpointer)dcc);
 	}
 
+#ifdef USE_DCC64
+	/* take the top 32 of "bytes send" and bottom 32 of "ack" */
+	dcc->ack = (dcc->pos & G_GINT64_CONSTANT (0xffffffff00000000)) |
+					(dcc->ack & 0xffffffff);
+	/* dcc->ack is only used for CPS and PERCENTAGE calcs from now on... */
+#endif
+
 	return TRUE;
 }
 
@@ -1018,13 +1085,7 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		break;
 
 	case TYPE_CHATSEND:
-		if (prefs.autodialog)
-		{
-			char *cmd = malloc (8 + strlen (dcc->nick));
-			sprintf (cmd, "query %s", dcc->nick);
-			handle_command (dcc->serv->server_session, cmd, FALSE);
-			free (cmd);
-		}
+		dcc_open_query (dcc->serv->server_session, dcc->nick);
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
@@ -1059,7 +1120,7 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 	SAddr.sin_family = AF_INET;
 
 	/*if local_ip is specified use that*/
-	if (prefs.local_ip != 0)
+	if (prefs.local_ip != 0xffffffff)
 	{
 		my_addr = prefs.local_ip;
 		SAddr.sin_addr.s_addr = prefs.local_ip;
@@ -1205,11 +1266,16 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 
 	if (stat (file_fs, &st) != -1)
 	{
+
+#ifndef USE_DCC64
 		if (sizeof (st.st_size) > 4 && st.st_size > 4294967295U)
 		{
 			PrintText (sess, "Cannot send files larger than 4 GB.\n");
-			goto noaxs;
+			g_free (file_fs);
+			dcc_close (dcc, 0, TRUE);
+			return;
 		}
+#endif
 
 		if (*file_part (file_fs) && !S_ISDIR (st.st_mode))
 		{
@@ -1251,15 +1317,15 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 						{
 							dcc->pasvid = new_id();
 							snprintf (outbuf, sizeof (outbuf), (havespaces) ?
-									"DCC SEND \"%s\" %lu %d %u %d" :
-									"DCC SEND %s %lu %d %u %d",
-									file_part (dcc->file), 199ul,
+									"DCC SEND \"%s\" 199 %d %"DCC_SFMT" %d" :
+									"DCC SEND %s 199 %d %"DCC_SFMT" %d",
+									file_part (dcc->file),
 									0, dcc->size, dcc->pasvid);
 						} else
 						{
 							snprintf (outbuf, sizeof (outbuf), (havespaces) ?
-									"DCC SEND \"%s\" %lu %d %u" :
-									"DCC SEND %s %lu %d %u",
+									"DCC SEND \"%s\" %lu %d %"DCC_SFMT :
+									"DCC SEND %s %lu %d %"DCC_SFMT,
 									file_part (dcc->file), dcc->addr,
 									dcc->port, dcc->size);
 						}
@@ -1277,7 +1343,6 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 		}
 	}
 	PrintTextf (sess, _("Cannot access %s\n"), dcc->file);
-noaxs:
 	g_free (file_fs);
 	dcc_close (dcc, 0, TRUE);
 }
@@ -1291,14 +1356,13 @@ find_dcc_from_id (int id, int type)
 	{
 		dcc = (struct DCC *) list->data;
 		if (dcc->pasvid == id &&
-		dcc->dccstat == STAT_QUEUED && dcc->type == type)
+			 dcc->dccstat == STAT_QUEUED && dcc->type == type)
 		return dcc;
 		list = list->next;
 	}
 	return 0;
 }
 
-
 static struct DCC *
 find_dcc_from_port (int port, int type)
 {
@@ -1431,7 +1495,7 @@ new_dcc (void)
 }
 
 void
-dcc_chat (struct session *sess, char *nick)
+dcc_chat (struct session *sess, char *nick, int passive)
 {
 	char outbuf[512];
 	struct DCC *dcc;
@@ -1475,7 +1539,7 @@ dcc_chat (struct session *sess, char *nick)
 	dcc->dccstat = STAT_QUEUED;
 	dcc->type = TYPE_CHATSEND;
 	dcc->nick = strdup (nick);
-	if (dcc_listen_init (dcc, sess))
+	if (passive || dcc_listen_init (dcc, sess))
 	{
 		if (prefs.autoopendccchatwindow)
 		{
@@ -1483,8 +1547,17 @@ dcc_chat (struct session *sess, char *nick)
 				fe_dcc_add (dcc);
 		} else
 			fe_dcc_add (dcc);
-		snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat %lu %d",
-						dcc->addr, dcc->port);
+
+		if (passive)
+		{
+			dcc->pasvid = new_id ();
+			snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat 199 %d %d",
+						 dcc->port, dcc->pasvid);
+		} else
+		{
+			snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat %lu %d",
+						 dcc->addr, dcc->port);
+		}
 		dcc->serv->p_ctcp (dcc->serv, nick, outbuf);
 		EMIT_SIGNAL (XP_TE_DCCCHATOFFERING, sess, nick, NULL, NULL, NULL, 0);
 	} else
@@ -1543,6 +1616,7 @@ is_resumable (struct DCC *dcc)
 			{
 				if (d != dcc && strcmp (d->destfile, dcc->destfile) == 0)
 				{
+					dcc->resume_error = 3;	/* dccgui.c uses it */
 					dcc->resumable = 0;
 					dcc->pos = 0;
 					break;
@@ -1564,8 +1638,8 @@ dcc_resume (struct DCC *dcc)
 	{
 		/* filename contains spaces? Quote them! */
 		snprintf (tbuf, sizeof (tbuf) - 10, strchr (dcc->file, ' ') ?
-					  "DCC RESUME \"%s\" %d %u" :
-					  "DCC RESUME %s %d %u",
+					  "DCC RESUME \"%s\" %d %"DCC_SFMT :
+					  "DCC RESUME %s %d %"DCC_SFMT,
 					  dcc->file, dcc->port, dcc->resumable);
 
 		if (dcc->pasvid)
@@ -1606,6 +1680,125 @@ dcc_deny_chat (void *ud)
 	dcc_abort (dcc->serv->front_session, dcc);
 }
 
+static struct DCC *
+dcc_add_chat (session *sess, char *nick, int port, unsigned long addr, int pasvid)
+{
+	struct DCC *dcc;
+
+	dcc = new_dcc ();
+	if (dcc)
+	{
+		dcc->serv = sess->server;
+		dcc->type = TYPE_CHATRECV;
+		dcc->dccstat = STAT_QUEUED;
+		dcc->addr = addr;
+		dcc->port = port;
+		dcc->pasvid = pasvid;
+		dcc->nick = strdup (nick);
+		dcc->starttime = time (0);
+
+		EMIT_SIGNAL (XP_TE_DCCCHATOFFER, sess->server->front_session, nick,
+						 NULL, NULL, NULL, 0);
+
+		if (prefs.autoopendccchatwindow)
+		{
+			if (fe_dcc_open_chat_win (TRUE))	/* already open? add only */
+				fe_dcc_add (dcc);
+		} else
+			fe_dcc_add (dcc);
+
+		if (prefs.autodccchat == 1)
+			dcc_connect (dcc);
+		else if (prefs.autodccchat == 2)
+		{
+			char buff[128];
+			snprintf (buff, sizeof (buff), "%s is offering DCC Chat.  Do you want to accept?", nick);
+			fe_confirm (buff, dcc_confirm_chat, dcc_deny_chat, dcc);
+		}
+	}
+
+	return dcc;
+}
+
+static struct DCC *
+dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, unsigned long addr, int pasvid)
+{
+	struct DCC *dcc;
+	char tbuf[512];
+
+	dcc = new_dcc ();
+	if (dcc)
+	{
+		dcc->file = strdup (file);
+
+		dcc->destfile = g_malloc (strlen (prefs.dccdir) + strlen (nick) +
+										  strlen (file) + 4);
+
+		strcpy (dcc->destfile, prefs.dccdir);
+		if (prefs.dccdir[strlen (prefs.dccdir) - 1] != '/')
+			strcat (dcc->destfile, "/");
+		if (prefs.dccwithnick)
+		{
+#ifdef WIN32
+			char *t = strlen (dcc->destfile) + dcc->destfile;
+			strcpy (t, nick);
+			while (*t)
+			{
+				if (*t == '\\' || *t == '|')
+					*t = '_';
+				t++;
+			}
+#else
+			strcat (dcc->destfile, nick);
+#endif
+			strcat (dcc->destfile, ".");
+		}
+		strcat (dcc->destfile, file);
+
+		/* get the local filesystem encoding */
+		dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
+
+		dcc->resumable = 0;
+		dcc->pos = 0;
+		dcc->serv = sess->server;
+		dcc->type = TYPE_RECV;
+		dcc->dccstat = STAT_QUEUED;
+		dcc->addr = addr;
+		dcc->port = port;
+		dcc->pasvid = pasvid;
+		dcc->size = size;
+		dcc->nick = strdup (nick);
+		dcc->maxcps = prefs.dcc_max_get_cps;
+
+		is_resumable (dcc);
+
+		/* autodccsend is really autodccrecv.. right? */
+
+		if (prefs.autodccsend == 1)
+		{
+			dcc_get (dcc);
+		}
+		else if (prefs.autodccsend == 2)
+		{
+			char buff[128];
+			snprintf (buff, sizeof (buff), "%s is offering \"%s\" via DCC.  Do you want to accept the transfer?", nick, file);
+			fe_confirm (buff, dcc_confirm_send, dcc_deny_send, dcc);
+		}
+		if (prefs.autoopendccrecvwindow)
+		{
+			if (fe_dcc_open_recv_win (TRUE))	/* was already open? just add*/
+				fe_dcc_add (dcc);
+		} else
+			fe_dcc_add (dcc);
+	}
+	sprintf (tbuf, "%"DCC_SFMT, size);
+	snprintf (tbuf + 24, 300, "%s:%d", net_ip (dcc->addr), dcc->port);
+	EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
+					 file, tbuf, tbuf + 24, 0);
+
+	return dcc;
+}
+
 void
 handle_dcc (struct session *sess, char *nick, char *word[],
 				char *word_eol[])
@@ -1614,7 +1807,9 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 	struct DCC *dcc;
 	char *type = word[5];
 	int port, pasvid = 0;
-	unsigned long size, addr;
+	unsigned long addr;
+	DCC_SIZE size;
+	int psend = 0;
 
 	if (!strcasecmp (type, "CHAT"))
 	{
@@ -1623,6 +1818,11 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 
 		if (port == 0)
 			pasvid = atoi (word[9]);
+		else if (word[9][0] != 0)
+		{
+			pasvid = atoi (word[9]);
+			psend = 1;
+		}
 
 		if (!addr /*|| (port < 1024 && port != 0)*/
 			|| port > 0xffff || (port == 0 && pasvid == 0))
@@ -1630,6 +1830,22 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc_malformed (sess, nick, word_eol[4] + 2);
 			return;
 		}
+
+		if (psend)
+		{
+			dcc = find_dcc_from_id (pasvid, TYPE_CHATSEND);
+			if (dcc)
+			{
+				dcc->addr = addr;
+				dcc->port = port;
+				dcc_connect (dcc);
+			} else
+			{
+				dcc_malformed (sess, nick, word_eol[4] + 2);
+			}
+			return;
+		}
+
 		dcc = find_dcc (nick, "", TYPE_CHATSEND);
 		if (dcc)
 			dcc_close (dcc, 0, TRUE);
@@ -1638,40 +1854,11 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 		if (dcc)
 			dcc_close (dcc, 0, TRUE);
 
-		dcc = new_dcc ();
-		if (dcc)
-		{
-			dcc->serv = sess->server;
-			dcc->type = TYPE_CHATRECV;
-			dcc->dccstat = STAT_QUEUED;
-			dcc->addr = addr;
-			dcc->port = port;
-			dcc->pasvid = pasvid;
-			dcc->nick = strdup (nick);
-			dcc->starttime = time (0);
-
-			EMIT_SIGNAL (XP_TE_DCCCHATOFFER, sess->server->front_session, nick,
-							 NULL, NULL, NULL, 0);
-
-			if (prefs.autoopendccchatwindow)
-			{
-				if (fe_dcc_open_chat_win (TRUE))	/* already open? add only */
-					fe_dcc_add (dcc);
-			} else
-				fe_dcc_add (dcc);
-
-			if (prefs.autodccchat == 1)
-				dcc_connect (dcc);
-			else if (prefs.autodccchat == 2)
-			{
-				char buff[128];
-				snprintf (buff, sizeof (buff), "%s is offering DCC Chat.  Do you want to accept?", nick);
-				fe_confirm (buff, dcc_confirm_chat, dcc_deny_chat, dcc);
-			}
-		}
+		dcc_add_chat (sess, nick, port, addr, pasvid);
 		return;
 	}
-	if (!strcasecmp (type, "RESUME"))
+
+	if (!strcasecmp (type, "Resume"))
 	{
 		port = atoi (word[7]);
 
@@ -1687,7 +1874,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc = find_dcc (nick, word[6], TYPE_SEND);
 		if (dcc)
 		{
-			size = strtoul (word[8], NULL, 10);
+			size = BIG_STR_TO_INT (word[8]);
 			dcc->resumable = size;
 			if (dcc->resumable < dcc->size)
 			{
@@ -1698,24 +1885,24 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 				/* Checking if dcc is passive and if filename contains spaces */
 				if (dcc->pasvid)
 					snprintf (tbuf, sizeof (tbuf), strchr (file_part (dcc->file), ' ') ?
-							"DCC ACCEPT \"%s\" %d %u %d" :
-							"DCC ACCEPT %s %d %u %d",
+							"DCC ACCEPT \"%s\" %d %"DCC_SFMT" %d" :
+							"DCC ACCEPT %s %d %"DCC_SFMT" %d",
 							file_part (dcc->file), port, dcc->resumable, dcc->pasvid);
 				else
 					snprintf (tbuf, sizeof (tbuf), strchr (file_part (dcc->file), ' ') ?
-							"DCC ACCEPT \"%s\" %d %u" :
-							"DCC ACCEPT %s %d %u",
+							"DCC ACCEPT \"%s\" %d %"DCC_SFMT :
+							"DCC ACCEPT %s %d %"DCC_SFMT,
 							file_part (dcc->file), port, dcc->resumable);
 
 				dcc->serv->p_ctcp (dcc->serv, dcc->nick, tbuf);
 			}
-			sprintf (tbuf, "%u", dcc->pos);
+			sprintf (tbuf, "%"DCC_SFMT, dcc->pos);
 			EMIT_SIGNAL (XP_TE_DCCRESUMEREQUEST, sess, nick,
 							 file_part (dcc->file), tbuf, NULL, 0);
 		}
 		return;
 	}
-	if (!strcasecmp (type, "ACCEPT"))
+	if (!strcasecmp (type, "Accept"))
 	{
 		port = atoi (word[7]);
 		dcc = find_dcc_from_port (port, TYPE_RECV);
@@ -1728,11 +1915,10 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 	if (!strcasecmp (type, "SEND"))
 	{
 		char *file = file_part (word[6]);
-		int psend = 0;
 
 		port = atoi (word[8]);
 		addr = strtoul (word[7], NULL, 10);
-		size = strtoul (word[9], NULL, 10);
+		size = BIG_STR_TO_INT (word[9]);
 
 		if (port == 0) /* Passive dcc requested */
 			pasvid = atoi (word[10]);
@@ -1778,78 +1964,13 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			return;
 		}
 
-		dcc = new_dcc ();
-		if (dcc)
-		{
-			dcc->file = strdup (file);
+		dcc_add_file (sess, file, size, port, nick, addr, pasvid);
 
-			dcc->destfile = g_malloc (strlen (prefs.dccdir) + strlen (nick) +
-											  strlen (file) + 4);
-
-			strcpy (dcc->destfile, prefs.dccdir);
-			if (prefs.dccdir[strlen (prefs.dccdir) - 1] != '/')
-				strcat (dcc->destfile, "/");
-			if (prefs.dccwithnick)
-			{
-#ifdef WIN32
-				char *t = strlen (dcc->destfile) + dcc->destfile;
-				strcpy (t, nick);
-				while (*t)
-				{
-					if (*t == '\\' || *t == '|')
-						*t = '_';
-					t++;
-				}
-#else
- 				strcat (dcc->destfile, nick);
-#endif
-				strcat (dcc->destfile, ".");
-			}
-			strcat (dcc->destfile, file);
-
-			/* get the local filesystem encoding */
-			dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
-
-			dcc->resumable = 0;
-			dcc->pos = 0;
-			dcc->serv = sess->server;
-			dcc->type = TYPE_RECV;
-			dcc->dccstat = STAT_QUEUED;
-			dcc->addr = addr;
-			dcc->port = port;
-			dcc->pasvid = pasvid;
-			dcc->size = size;
-			dcc->nick = strdup (nick);
-			dcc->maxcps = prefs.dcc_max_get_cps;
-
-			is_resumable (dcc);
-
-			/* autodccsend is really autodccrecv.. right? */
-
-			if (prefs.autodccsend == 1)
-			{
-				dcc_get (dcc);
-			}
-			else if (prefs.autodccsend == 2)
-			{
-				char buff[128];
-				snprintf (buff, sizeof (buff), "%s is offering \"%s\" via DCC.  Do you want to accept the transfer?", nick, file);
-				fe_confirm (buff, dcc_confirm_send, dcc_deny_send, dcc);
-			}
-			if (prefs.autoopendccrecvwindow)
-			{
-				if (fe_dcc_open_recv_win (TRUE))	/* was already open? just add*/
-					fe_dcc_add (dcc);
-			} else
-				fe_dcc_add (dcc);
-		}
-		sprintf (tbuf, "%lu", size);
-		snprintf (tbuf + 24, 300, "%s:%d", net_ip (dcc->addr), dcc->port);
-		EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
-						 file, tbuf, tbuf + 24, 0);
 	} else
+	{
 		EMIT_SIGNAL (XP_TE_DCCGENERICOFFER, sess->server->front_session,
 						 word_eol[4] + 2, nick, NULL, NULL, 0);
+	}
 }
 
 void
@@ -1864,7 +1985,7 @@ dcc_show_list (struct session *sess)
 	{
 		dcc = (struct DCC *) list->data;
 		i++;
-		PrintTextf (sess, " %s  %-10.10s %-7.7s %-7u %-7u %s\n",
+		PrintTextf (sess, " %s  %-10.10s %-7.7s %-7"DCC_SFMT" %-7"DCC_SFMT" %s\n",
 					 dcctypes[dcc->type], dcc->nick,
 					 _(dccstat[dcc->dccstat].name), dcc->size, dcc->pos,
 					 file_part (dcc->file));
