@@ -2,6 +2,9 @@
 
 #include <time.h>						/* for time_t */
 
+#ifndef XCHAT_DCC_H
+#define XCHAT_DCC_H
+
 #define STAT_QUEUED 0
 #define STAT_ACTIVE 1
 #define STAT_FAILED 2
@@ -16,6 +19,17 @@
 
 #define CPS_AVG_WINDOW 10
 
+/* can we do 64-bit dcc? */
+#if defined(G_GINT64_FORMAT) && defined(HAVE_STRTOULL)
+#define USE_DCC64
+/* we really get only 63 bits, since st_size is signed */
+#define DCC_SIZE gint64
+#define DCC_SFMT G_GINT64_FORMAT
+#else
+#define DCC_SIZE unsigned int
+#define DCC_SFMT "u"
+#endif
+
 struct DCC
 {
 	struct server *serv;
@@ -32,13 +46,13 @@ struct DCC
 	int resume_errno;
 
 	GTimeVal lastcpstv, firstcpstv;
-	unsigned int lastcpspos;
+	DCC_SIZE lastcpspos;
 	int maxcps;
 
-	unsigned int size;
-	unsigned int resumable;
-	unsigned int ack;
-	unsigned int pos;
+	DCC_SIZE size;
+	DCC_SIZE resumable;
+	DCC_SIZE ack;
+	DCC_SIZE pos;
 	time_t starttime;
 	time_t offertime;
 	time_t lasttime;
@@ -76,12 +90,14 @@ void dcc_check_timeouts (void);
 void dcc_change_nick (server *serv, char *oldnick, char *newnick);
 void dcc_notify_kill (struct server *serv);
 struct DCC *dcc_write_chat (char *nick, char *text);
-void dcc_send (struct session *sess, char *tbuf, char *to, char *file, int maxcps);
+void dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive);
 struct DCC *find_dcc (char *nick, char *file, int type);
 void dcc_get_nick (struct session *sess, char *nick);
-void dcc_chat (session *sess, char *nick);
+void dcc_chat (session *sess, char *nick, int passive);
 void handle_dcc (session *sess, char *nick, char *word[], char *word_eol[]);
 void dcc_show_list (session *sess);
 void open_dcc_recv_window (void);
 void open_dcc_send_window (void);
 void open_dcc_chat_window (void);
+
+#endif
