@@ -218,7 +218,7 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 static int
 SearchNick (char *text, char *nicks)
 {
-	char S[300];	/* size of bluestring in xchatprefs */
+	char S[300];	/* size of irc_extra_hilight in xchatprefs */
 	char *n;
 	char *p;
 	char *t;
@@ -227,7 +227,7 @@ SearchNick (char *text, char *nicks)
 	if (nicks == NULL)
 		return 0;
 
-	text = strip_color (text);
+	text = strip_color (text, -1, 1, 1);
 
 	safe_strcpy (S, nicks, sizeof (S));
 	n = strtok (S, ",");
@@ -257,21 +257,40 @@ SearchNick (char *text, char *nicks)
 }
 
 static int
-is_hilight (char *text, session *sess, server *serv)
+FromNick (char *nick, char *nicks)
 {
-	if ((SearchNick (text, serv->nick)) || SearchNick (text, prefs.bluestring))
+	char S[300];	/* size of irc_no_hilight in xchatprefs */
+	char *n;
+	char *t;
+
+	if (nicks == NULL || nicks[0] == 0)
+		return 0;
+
+	safe_strcpy (S, nicks, sizeof (S));
+	n = strtok (S, ",");
+	while (n != NULL)
+	{
+		t = nick;
+		if (nocasestrstr(t, n))
+			return 1;
+		n = strtok (NULL, ",");
+	}
+	return 0;
+}
+
+static int
+is_hilight (char *from, char *text, session *sess, server *serv)
+{
+	if (FromNick(from, prefs.irc_no_hilight))
+		return 0;
+
+	if (SearchNick (text, serv->nick) ||
+		 SearchNick (text, prefs.irc_extra_hilight) ||
+		 FromNick (from, prefs.irc_nick_hilight))
 	{
-#ifdef WIN32
 		if (sess != current_tab)
 			sess->nick_said = TRUE;
 		fe_set_hilight (sess);
-#else
-		if (sess != current_tab)
-		{
-			sess->nick_said = TRUE;
-			fe_set_hilight (sess);
-		}
-#endif
 		return 1;
 	}
 	return 0;
@@ -321,7 +340,7 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 
 	if (!fromme)
 	{
-		hilight = is_hilight (text, sess, serv);
+		hilight = is_hilight (from, text, sess, serv);
 		if (hilight && prefs.beephilight)
 			beep = TRUE;
 
@@ -404,7 +423,7 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 		if (prefs.beepchans || sess->beep)
 			sound_beep (sess);
 
-	if (is_hilight (text, sess, serv))
+	if (is_hilight (from, text, sess, serv))
 	{
 		hilight = TRUE;
 		if (prefs.beephilight)
@@ -643,7 +662,7 @@ inbound_topic (server *serv, char *chan, char *topic_text)
 
 	if (sess)
 	{
-		new_topic = strip_color (topic_text);
+		new_topic = strip_color (topic_text, -1, 1, 1);
 		set_topic (sess, new_topic);
 		free (new_topic);
 	} else
@@ -661,7 +680,7 @@ inbound_topicnew (server *serv, char *nick, char *chan, char *topic)
 	sess = find_channel (serv, chan);
 	if (sess)
 	{
-		new_topic = strip_color (topic);
+		new_topic = strip_color (topic, -1, 1, 1);
 		set_topic (sess, new_topic);
 		free (new_topic);
 		EMIT_SIGNAL (XP_TE_NEWTOPIC, sess, nick, topic, chan, NULL, 0);
@@ -857,6 +876,12 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 		sess = find_channel (serv, ptr);
 	}
 
+	if (!sess && ptr[0] == '%')
+	{
+		ptr++;
+		sess = find_channel (serv, ptr);
+	}
+
 	if (!sess && ptr[0] == '+')
 	{
 		ptr++;
@@ -1192,8 +1217,8 @@ inbound_user_info (session *sess, char *chan, char *user, char *host,
 	return 1;
 }
 
-void
-inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *banner)
+int
+inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *banner, int is_exemption)
 {
 	char *time_str = ctime (&stamp);
 	server *serv = sess->server;
@@ -1206,9 +1231,17 @@ inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *bann
 	if (!sess)
 		sess = serv->front_session;
    if (!fe_is_banwindow (sess))
+	{
+		/* let proto-irc.c do the 'goto def' for exemptions */
+		if (is_exemption)
+			return FALSE;
+
 		EMIT_SIGNAL (XP_TE_BANLIST, sess, chan, mask, banner, time_str, 0);
-	else
-		fe_add_ban_list (sess, mask, banner, time_str);
+		return TRUE;
+	}
+
+	fe_add_ban_list (sess, mask, banner, time_str, is_exemption);
+	return TRUE;
 }
 
 /* execute 1 end-of-motd command */
