@@ -2,6 +2,7 @@
 #define XCHAT_OUTBOUND_H
 
 extern const struct commands xc_cmds[];
+extern GSList *menu_list;
 
 int load_trans_table (char *full_path);
 int auto_insert (char *dest, int destlen, unsigned char *src, char *word[], char *word_eol[],
@@ -14,5 +15,6 @@ void check_special_chars (char *cmd, int do_ascii);
 void notc_msg (session *sess);
 void server_sendpart (server * serv, char *channel, char *reason);
 void server_sendquit (session * sess);
+int menu_streq (const char *s1, const char *s2, int def);
 
 #endif
