@@ -268,8 +268,8 @@ server_inline (server *serv, char *line, int len)
 	/* Checks whether we're set to use UTF-8 charset */
 	if ((serv->encoding == NULL && prefs.utf8_locale) ||
 	    (serv->encoding != NULL &&
-		 (strcasecmp (serv->encoding, "utf8") == 0 ||
-		  strcasecmp (serv->encoding, "utf-8") == 0)))
+		 (strcasecmp (serv->encoding, "UTF8") == 0 ||
+		  strcasecmp (serv->encoding, "UTF-8") == 0)))
 	{
 		/* The user has the UTF-8 charset set, either via /charset
 		command or from his UTF-8 locale. Thus, we first try the
@@ -319,7 +319,7 @@ server_inline (server *serv, char *line, int len)
 				utf_line_allocated = g_convert_with_fallback (conv_line, conv_len, "UTF-8", encoding, "?", &read_len, &utf_len, &err);
 				if (err != NULL)
 				{
-					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE)
+					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE && conv_len > (read_len + 1))
 					{
 						/* Make our best bet by removing the erroneous char.
 						   This will work for casual 8-bit strings with non-standard chars. */
@@ -351,7 +351,7 @@ server_inline (server *serv, char *line, int len)
 	}
 
 	fe_add_rawlog (serv, line, len, FALSE);
-	url_check (line);
+	url_check_line (line, len);
 
 	/* let proto-irc.c handle it */
 	serv->p_inline (serv, line, len);
@@ -385,7 +385,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 			error = 0;
 			if (len < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
 					return TRUE;
 				error = sock_error ();
 			}
