@@ -31,6 +31,7 @@
 #include "cfgfiles.h"
 #include "plugin.h"
 #include "fe.h"
+#include "server.h"
 #include "util.h"
 #include "outbound.h"
 #include "xchatc.h"
@@ -47,153 +48,6 @@ struct pevt_stage1
 };
 
 
-/* Make ip:port urls clikable (127.0.0.1:80 etc)
-*  patch by Alex <alex@cosinus.org> & dobler <dobler@barrysworld.com>
-*/
-
-static int
-q3link (char *word)
-{
-	char *s;
-	int i;
-	int d = 0;
-
-	if ((s = strchr (word,':')) != NULL)
-	{
-		for (i = 0; i < s - word; i++)
-		{
-			if (word[i] == '.')
-				d++;
-			else if (!isdigit (word[i]))
-			{
-				d = 0;
-				break;
-			}
-		}
-
-		if (d == 3)
-		{
-			s++;
-			d = 0;
-
-			while (*s != 0)
-			{
-				if (!isdigit (*s++))
-					return (0);
-				d++;
-			}
-			if (d > 0)
-				return (1);
-		}
-	}
-	return (0);
-}
-
-/* check if a word is clickable */
-
-int
-text_word_check (char *word)
-{
-	session *sess = current_sess;
-	char *at, *dot;
-	int i, dots;
-	int len = strlen (word);
-
-	if(q3link(word))
-		return WORD_URL;
-
-	if ((word[0] == '@' || word[0] == '+' || word[0] == '^' || word[0] == '%' || word[0] == '*' ) && word[1] == '#')
-		return WORD_CHANNEL;
-
-	if ((word[0] == '#' || word[0] == '&') && word[1] != '#' && word[1] != 0)
-		return WORD_CHANNEL;
-
-	if (!strncasecmp (word, "irc.", 4) && word[4] != '.')
-		return WORD_URL;
-
-	if (!strncasecmp (word, "ftp.", 4) && word[4] != '.')
-		return WORD_URL;
-
-	if (!strncasecmp (word, "www.", 4) && word[4] != '.')
-		return WORD_URL;
-
-	if (!strncasecmp (word, "irc://", 6) && word[6] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "ftp://", 6) && word[6] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "http://", 7) && word[7] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "file://", 7) && word[7] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "https://", 8) && word[8] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "gopher://", 9) && word[9] != 0)
-		return WORD_URL;
-
-	if (( (word[0]=='@' || word[0]=='+') && find_name (sess, word+1)) || find_name (sess, word))
-		return WORD_NICK;
-
-	at = strchr (word, '@');	  /* check for email addy */
-	dot = strrchr (word, '.');
-	if (at && dot)
-	{
-		if (at < dot)
-		{
-			if (strchr (word, '*'))
-				return WORD_HOST;
-			else
-				return WORD_EMAIL;
-		}
-	}
- 
-	/* check if it's an IP number */
-	dots = 0;
-	for (i = 0; i < len; i++)
-	{
-		if (word[i] == '.' && i > 1)
-			dots++;
-		else if (!isdigit (word[i]))
-		{
-			dots = 0;
-			break;
-		}
-	}
-	if (dots == 3)
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 5, ".html", 5))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".org", 4))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".net", 4))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".com", 4))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".edu", 4))
-		return WORD_HOST;
-
-	if (len > 5)
-	{
-		if (word[len - 3] == '.' &&
-			 isalpha (word[len - 2]) && isalpha (word[len - 1]))
-			return WORD_HOST;
-	}
-
-	if (sess->type == SESS_DIALOG)
-		return WORD_DIALOG;
-
-	return 0;
-}
-
 void
 log_close (session *sess)
 {
@@ -335,7 +189,7 @@ log_open (session *sess)
 
 	log_close (sess);
 	sess->logfd = log_open_file (sess->server->servername, sess->channel,
-										  get_network (sess, FALSE));
+										  server_get_network (sess->server, FALSE));
 
 	if (!log_error && sess->logfd == -1)
 	{
@@ -401,7 +255,11 @@ log_write (session *sess, char *text)
 			}
 		}
 		temp = strip_color (text, -1, 1, 1);
-		write (sess->logfd, temp, strlen (temp));
+		len = strlen (temp);
+		write (sess->logfd, temp, len);
+		/* lots of scripts/plugins print without a \n at the end */
+		if (temp[len - 1] != '\n')
+			write (sess->logfd, "\n", 1);	/* emulate what xtext would display */
 		free (temp);
 	}
 }
@@ -673,6 +531,7 @@ static char * const pevt_partreason_help[] = {
 static char * const pevt_ctcpsnd_help[] = {
 	N_("The sound"),
 	N_("The nick of the person"),
+	N_("The channel"),
 };
 
 static char * const pevt_ctcpgen_help[] = {
@@ -870,7 +729,7 @@ static char * const pevt_generic_nick_help[] = {
 };
 
 static char * const pevt_chanmodes_help[] = {
-	N_("Channel name"),
+	N_("Channel Name"),
 	N_("Modes string"),
 };
 
@@ -983,7 +842,7 @@ static char * const pevt_notifynumber_help[] = {
 };
 
 static char * const pevt_serverlookup_help[] = {
-	N_("Servername"),
+	N_("Server Name"),
 };
 
 static char * const pevt_servererror_help[] = {
@@ -1275,13 +1134,17 @@ load_text_events ()
 	pevent_make_pntevts ();
 }
 
-static void
-display_event (char *i, session *sess, int numargs, char **args)
+void
+format_event (session *sess, int index, char **args, char *o, int sizeofo)
 {
-	int len, oi, ii;
-	char *ar, o[4096], d, a, done_all = FALSE;
+	int len, oi, ii, numargs;
+	char *i, *ar, d, a, done_all = FALSE;
+
+	i = pntevts[index];
+	numargs = te[index].num_args;
 
 	oi = ii = len = d = a = 0;
+	o[0] = 0;
 
 	if (i == NULL)
 		return;
@@ -1294,9 +1157,10 @@ display_event (char *i, session *sess, int numargs, char **args)
 		case 0:
 			memcpy (&len, &(i[ii]), sizeof (int));
 			ii += sizeof (int);
-			if (oi + len > sizeof (o))
+			if (oi + len > sizeofo)
 			{
 				printf ("Overflow in display_event (%s)\n", i);
+				o[0] = 0;
 				return;
 			}
 			memcpy (&(o[oi]), &(i[ii]), len);
@@ -1347,8 +1211,16 @@ display_event (char *i, session *sess, int numargs, char **args)
 	}
 	o[oi] = 0;
 	if (*o == '\n')
-		return;
-	PrintText (sess, o);
+		o[0] = 0;
+}
+
+static void
+display_event (session *sess, int event, char **args)
+{
+	char o[4096];
+	format_event (sess, event, args, o, sizeof (o));
+	if (o[0])
+		PrintText (sess, o);
 }
 
 int
@@ -1433,7 +1305,7 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 			fe_message ("String ends in $a", FALSE);
 			return 1;
 		 a_range_error:
-			fe_message ("$a value is greater then 255", FALSE);
+			fe_message ("$a value is greater than 255", FALSE);
 			return 1;
 		}
 		if (d == 't')
@@ -1557,7 +1429,9 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 
 	sound_play_event (index);
 
-	display_event (pntevts[index], sess, te[index].num_args, word);
+	/* If a plugin's callback executes "/close", 'sess' may be invalid */
+	if (is_session (sess))
+		display_event (sess, index, word);
 }
 
 int
