@@ -115,7 +115,7 @@ xchat_realloc (char *old, int len, char *file, int line)
 	if (ret)
 	{
 		strcpy (ret, old);
-		xchat_free (old, file, line);
+		xchat_dfree (old, file, line);
 	}
 	return ret;
 }
@@ -193,7 +193,7 @@ xchat_mem_list (void)
 }
 
 void
-xchat_free (void *buf, char *file, int line)
+xchat_dfree (void *buf, char *file, int line)
 {
 	struct mem_block *cur, *last;
 
@@ -233,7 +233,7 @@ xchat_free (void *buf, char *file, int line)
 
 #define malloc(n) xchat_malloc(n, __FILE__, __LINE__)
 #define realloc(n, m) xchat_realloc(n, m, __FILE__, __LINE__)
-#define free(n) xchat_free(n, __FILE__, __LINE__)
+#define free(n) xchat_dfree(n, __FILE__, __LINE__)
 #define strdup(n) xchat_strdup(n, __FILE__, __LINE__)
 
 #endif /* MEMORY_DEBUG */
@@ -421,13 +421,17 @@ expand_homedir (char *file)
 }
 
 char *
-strip_color (char *text)
+strip_color (char *text, int len, int do_color, int do_attr)
 {
 	int nc = 0;
 	int i = 0;
 	int col = 0;
-	int len = strlen (text);
-	char *new_str = malloc (len + 2);
+	char *new_str;
+
+	if (len == -1)
+		len = strlen (text);
+
+	new_str = malloc (len + 2);
 
 	while (len > 0)
 	{
@@ -437,6 +441,11 @@ strip_color (char *text)
 			nc++;
 			if (*text == ',')
 				nc = 0;
+			if (!do_color)
+			{
+				new_str[i] = *text;
+				i++;
+			}
 		} else
 		{
 			col = 0;
@@ -445,12 +454,22 @@ strip_color (char *text)
 			case '\003':			  /*ATTR_COLOR: */
 				col = 1;
 				nc = 0;
+				if (!do_color)
+				{
+					new_str[i] = *text;
+					i++;
+				}
 				break;
 			case '\007':			  /*ATTR_BEEP: */
 			case '\017':			  /*ATTR_RESET: */
 			case '\026':			  /*ATTR_REVERSE: */
 			case '\002':			  /*ATTR_BOLD: */
 			case '\037':			  /*ATTR_UNDERLINE: */
+				if (!do_attr)
+				{
+					new_str[i] = *text;
+					i++;
+				}
 				break;
 			default:
 				new_str[i] = *text;
@@ -786,25 +805,19 @@ tolowerStr (char *str)
 	}
 }*/
 
-/* This is originally from BitchX, but since added missing TLDs and
-   changed to a binary search */
-
 typedef struct
 {
-	char *code;
-	char *country;
-} Domain;
+	char *code, *country;
+} domain_t;
 
 static int
 country_compare (const void *a, const void *b)
 {
-	return strcasecmp (a, ((Domain *)b)->code);
+	return strcasecmp (a, ((domain_t *)b)->code);
 }
 
-char *
-country (char *hostname)
+static const domain_t domain[] =
 {
-	static const Domain domain[] = {
 		{"AD", N_("Andorra") },
 		{"AE", N_("United Arab Emirates") },
 		{"AF", N_("Afghanistan") },
@@ -845,6 +858,7 @@ country (char *hostname)
 		{"BZ", N_("Belize") },
 		{"CA", N_("Canada") },
 		{"CC", N_("Cocos Islands") },
+		{"CD", N_("Democratic Republic of Congo") },
 		{"CF", N_("Central African Republic") },
 		{"CG", N_("Congo") },
 		{"CH", N_("Switzerland") },
@@ -856,7 +870,7 @@ country (char *hostname)
 		{"CO", N_("Colombia") },
 		{"COM", N_("Internic Commercial") },
 		{"CR", N_("Costa Rica") },
-		{"CS", N_("Former Czechoslovakia") },
+		{"CS", N_("Serbia and Montenegro") },
 		{"CU", N_("Cuba") },
 		{"CV", N_("Cape Verde") },
 		{"CX", N_("Christmas Island") },
@@ -994,6 +1008,7 @@ country (char *hostname)
 		{"PM", N_("St. Pierre and Miquelon") },
 		{"PN", N_("Pitcairn") },
 		{"PR", N_("Puerto Rico") },
+		{"PS", N_("Palestinian Territory") },
 		{"PT", N_("Portugal") },
 		{"PW", N_("Palau") },
 		{"PY", N_("Paraguay") },
@@ -1060,11 +1075,14 @@ country (char *hostname)
 		{"YU", N_("Yugoslavia") },
 		{"ZA", N_("South Africa") },
 		{"ZM", N_("Zambia") },
-		{"ZR", N_("Zaire") },
 		{"ZW", N_("Zimbabwe") },
-	};
+};
+
+char *
+country (char *hostname)
+{
 	char *p;
-	Domain *dom;
+	domain_t *dom;
 
 	if (!hostname || !*hostname || isdigit (hostname[strlen (hostname) - 1]))
 		return _("Unknown");
@@ -1073,8 +1091,8 @@ country (char *hostname)
 	else
 		p = hostname;
 
-	dom = bsearch (p, domain, sizeof (domain) / sizeof (Domain),
-						sizeof (Domain), country_compare);
+	dom = bsearch (p, domain, sizeof (domain) / sizeof (domain_t),
+						sizeof (domain_t), country_compare);
 
 	if (!dom)
 		return _("Unknown");
@@ -1082,6 +1100,22 @@ country (char *hostname)
 	return _(dom->country);
 }
 
+void
+country_search (char *pattern, void *ud, void (*print)(void *, char *, ...))
+{
+	const domain_t *dom;
+	int i;
+
+	for (i = 0; i < sizeof (domain) / sizeof (domain_t); i++)
+	{
+		dom = &domain[i];
+		if (match (pattern, dom->country) || match (pattern, _(dom->country)))
+		{
+			print (ud, "%s = %s\n", dom->code, _(dom->country));
+		}
+	}
+}
+
 /* I think gnome1.0.x isn't necessarily linked against popt, ah well! */
 /* !!! For now use this inlined function, or it would break fe-text building */
 /* .... will find a better solution later. */
@@ -1182,6 +1216,7 @@ util_exec (char *cmd)
 	int pid;
 	char **argv;
 	int argc;
+	int fd;
 
 	if (my_poptParseArgvString (cmd, &argc, &argv) != 0)
 		return -1;
@@ -1192,6 +1227,8 @@ util_exec (char *cmd)
 		return -1;
 	if (pid == 0)
 	{
+		/* Now close all open file descriptors except stdin, stdout and stderr */
+		for (fd = 3; fd < 1024; fd++) close(fd);
 		execvp (argv[0], argv);
 		_exit (0);
 	} else
@@ -1522,3 +1559,40 @@ mkdir_utf8 (char *dir)
 
 	return ret;
 }
+
+/* separates a string according to a 'sep' char, then calls the callback
+   function for each token found */
+
+int
+token_foreach (char *str, char sep,
+					int (*callback) (char *str, void *ud), void *ud)
+{
+	char t, *start = str;
+
+	while (1)
+	{
+		if (*str == sep || *str == 0)
+		{
+			t = *str;
+			*str = 0;
+			if (callback (start, ud) < 1)
+			{
+				*str = t;
+				return FALSE;
+			}
+			*str = t;
+
+			if (*str == 0)
+				break;
+			str++;
+			start = str;
+
+		} else
+		{
+			/* chars $00-$7f can never be embedded in utf-8 */
+			str++;
+		}
+	}
+
+	return TRUE;
+}
