@@ -194,7 +194,7 @@ away_check (void)
 	GSList *list;
 	int full, sent, loop = 0;
 
-	if (prefs.away_size_max < 1)
+	if (!prefs.away_track || prefs.away_size_max < 1)
 		return 1;
 
 doover:
@@ -305,7 +305,7 @@ irc_init (session *sess)
 }
 
 static session *
-new_session (server *serv, char *from, int type)
+new_session (server *serv, char *from, int type, int focus)
 {
 	session *sess;
 
@@ -322,89 +322,35 @@ new_session (server *serv, char *from, int type)
 
 	sess_list = g_slist_prepend (sess_list, sess);
 
-	fe_new_window (sess);
+	fe_new_window (sess, focus);
 
 	return sess;
 }
 
-void
-set_server_defaults (server *serv)
-{
-	if (serv->chantypes)
-		free (serv->chantypes);
-	if (serv->chanmodes)
-		free (serv->chanmodes);
-	if (serv->nick_prefixes)
-		free (serv->nick_prefixes);
-	if (serv->nick_modes)
-		free (serv->nick_modes);
-	/*if (serv->encoding)
-	{
-		free (serv->encoding);
-		serv->encoding = NULL;
-	}*/
-
-	serv->chantypes = strdup ("#&!+");
-	serv->chanmodes = strdup ("beI,k,l");
-	serv->nick_prefixes = strdup ("@%+");
-	serv->nick_modes = strdup ("ohv");
-
-	serv->nickcount = 1;
-	serv->end_of_motd = FALSE;
-	serv->is_away = FALSE;
-	serv->supports_watch = FALSE;
-	serv->bad_prefix = FALSE;
-	serv->use_who = TRUE;
-}
-
-static server *
-new_server (void)
-{
-	static int id = 0;
-	server *serv;
-
-	serv = malloc (sizeof (struct server));
-	memset (serv, 0, sizeof (struct server));
-
-	/* use server.c and proto-irc.c functions */
-	server_fill_her_up (serv);
-
-	serv->id = id++;
-	serv->sok = -1;
-	strcpy (serv->nick, prefs.nick1);
-	set_server_defaults (serv);
-
-	serv_list = g_slist_prepend (serv_list, serv);
-
-	fe_new_server (serv);
-
-	return serv;
-}
-
 session *
-new_ircwindow (server *serv, char *name, int type)
+new_ircwindow (server *serv, char *name, int type, int focus)
 {
 	session *sess;
 
 	switch (type)
 	{
 	case SESS_SERVER:
-		serv = new_server ();
+		serv = server_new ();
 		if (prefs.use_server_tab)
 		{
 			register unsigned int oldh = prefs.hideuserlist;
 			prefs.hideuserlist = 1;
-			sess = new_session (serv, name, SESS_SERVER);
+			sess = new_session (serv, name, SESS_SERVER, focus);
 			prefs.hideuserlist = oldh;
 		} else
 		{
-			sess = new_session (serv, name, SESS_CHANNEL);
+			sess = new_session (serv, name, SESS_CHANNEL, focus);
 		}
 		serv->server_session = sess;
 		serv->front_session = sess;
 		break;
 	case SESS_DIALOG:
-		sess = new_session (serv, name, type);
+		sess = new_session (serv, name, type, focus);
 		if (prefs.logging)
 			log_open (sess);
 		break;
@@ -412,7 +358,7 @@ new_ircwindow (server *serv, char *name, int type)
 /*	case SESS_CHANNEL:
 	case SESS_NOTICES:
 	case SESS_SNOTICES:*/
-		sess = new_session (serv, name, type);
+		sess = new_session (serv, name, type, focus);
 		break;
 	}
 
@@ -488,14 +434,6 @@ kill_server_callback (server * serv)
 		free (serv->bad_nick_prefixes);
 	if (serv->last_away_reason)
 		free (serv->last_away_reason);
-	if (serv->eom_cmd)
-		free (serv->eom_cmd);
-	if (serv->username)
-		free (serv->username);
-	if (serv->realname)
-		free (serv->realname);
-	if (serv->networkname)
-		free (serv->networkname);
 	if (serv->encoding)
 		free (serv->encoding);
 
@@ -675,16 +613,16 @@ find_away_message (struct server *serv, char *nick)
 
 #define XTERM "gnome-terminal -x "
 
-#define defaultconf_ctcp \
+static char defaultconf_ctcp[] =
 	"NAME TIME\n"				"CMD nctcp %s TIME %t\n\n"\
-	"NAME PING\n"				"CMD nctcp %s PING %d\n\n"
+	"NAME PING\n"				"CMD nctcp %s PING %d\n\n";
 
-#define defaultconf_replace \
-	"NAME teh\n"				"CMD the\n\n"
+static char defaultconf_replace[] =
+	"NAME teh\n"				"CMD the\n\n";
 /*	"NAME r\n"					"CMD are\n\n"\
 	"NAME u\n"					"CMD you\n\n"*/
 
-#define defaultconf_commands \
+static char defaultconf_commands[] =
 	"NAME ACTION\n"		"CMD me &2\n\n"\
 	"NAME AME\n"			"CMD allchan me &2\n\n"\
 	"NAME ANICK\n"			"CMD allserv nick &2\n\n"\
@@ -711,13 +649,13 @@ find_away_message (struct server *serv, char *nick)
 	"NAME VER\n"			"CMD ctcp %2 VERSION\n\n"\
 	"NAME VERSION\n"		"CMD ctcp %2 VERSION\n\n"\
 	"NAME WALLOPS\n"		"CMD quote WALLOPS :&2\n\n"\
-	"NAME WII\n"			"CMD quote WHOIS %2 %2\n\n"
+	"NAME WII\n"			"CMD quote WHOIS %2 %2\n\n";
 
 #ifdef WIN32
-#define defaultconf_urlhandlers \
-	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
+static char defaultconf_urlhandlers[] =
+	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n";
 #else
-#define defaultconf_urlhandlers \
+static char defaultconf_urlhandlers[] =
 	"NAME SUB\n"								"CMD Epiphany...\n\n"\
 		"NAME Open\n"							"CMD !epiphany '%s'\n\n"\
 		"NAME Open in new tab\n"			"CMD !epiphany -n '%s'\n\n"\
@@ -735,9 +673,9 @@ find_away_message (struct server *serv, char *nick)
 		"NAME Run new Mozilla\n"			"CMD !mozilla %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Mozilla FireFox...\n\n"\
-		"NAME Open in existing\n"			"CMD !firefox -remote 'openURL(%s)'\n\n"\
-		"NAME Open in new window\n"		"CMD !firefox -remote 'openURL(%s,new-window)'\n\n"\
-		"NAME Open in new tab\n"			"CMD !firefox -remote 'openURL(%s,new-tab)'\n\n"\
+		"NAME Open in existing\n"			"CMD !firefox -a firefox -remote 'openURL(%s)'\n\n"\
+		"NAME Open in new window\n"		"CMD !firefox -a firefox -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Open in new tab\n"			"CMD !firefox -a firefox -remote 'openURL(%s,new-tab)'\n\n"\
 		"NAME Run new Mozilla FireFox\n"	"CMD !firefox %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Galeon...\n\n"\
@@ -753,7 +691,7 @@ find_away_message (struct server *serv, char *nick)
 		"NAME Run new Opera\n"				"CMD !opera %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Send URL to...\n\n"\
-		"NAME Gnome URL Handler\n"			"CMD !gnome-moz-remote %s\n\n"\
+		"NAME Gnome URL Handler\n"			"CMD !gnome-open %s\n\n"\
 		"NAME Lynx\n"							"CMD !"XTERM"lynx %s\n\n"\
 		"NAME Links\n"							"CMD !"XTERM"links %s\n\n"\
 		"NAME w3m\n"							"CMD !"XTERM"w3m %s\n\n"\
@@ -763,7 +701,7 @@ find_away_message (struct server *serv, char *nick)
 		"NAME Telnet\n"						"CMD !"XTERM"telnet %s\n\n"\
 		"NAME Ping\n"							"CMD !"XTERM"ping -c 4 %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
-	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
+	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n";
 #endif
 
 #ifdef USE_SIGACTION
@@ -808,7 +746,7 @@ xchat_auto_connect (gpointer userdata)
 static void
 xchat_init (void)
 {
-	char buf[2048];
+	char buf[3068];
 	const char *cs = NULL;
 
 #ifdef WIN32
@@ -855,6 +793,7 @@ xchat_init (void)
 		prefs.utf8_locale = TRUE;
 
 	load_text_events ();
+	sound_load ();
 	notify_load ();
 	ignore_load ();
 
@@ -1034,7 +973,7 @@ xchat_init (void)
 			/* and no serverlist gui ... */
 			if (prefs.slist_skip || connect_url)
 				/* we'll have to open one. */
-				new_ircwindow (NULL, NULL, SESS_SERVER);
+				new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 		} else
 		{
 			fe_idle_add (xchat_auto_connect, NULL);
@@ -1042,7 +981,7 @@ xchat_init (void)
 	} else
 	{
 		if (prefs.slist_skip)
-			new_ircwindow (NULL, NULL, SESS_SERVER);
+			new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	}
 }
 
@@ -1056,10 +995,12 @@ xchat_exit (void)
 	if (prefs.autosave)
 	{
 		save_config ();
-		pevent_save (NULL);
+		if (prefs.save_pevents)
+			pevent_save (NULL);
 	}
 	if (prefs.autosave_url)
 		url_autosave ();
+	sound_save ();
 	notify_save ();
 	ignore_save ();
 	free_sessions ();
