@@ -36,6 +36,7 @@
 #include <gtk/gtktreeselection.h>
 
 #include "../common/xchat.h"
+#include "../common/fe.h"
 #include "../common/modes.h"
 #include "../common/outbound.h"
 #include "../common/xchatc.h"
@@ -65,19 +66,23 @@ get_store (struct session *sess)
 }
 
 void
-fe_add_ban_list (struct session *sess, char *mask, char *who, char *when)
+fe_add_ban_list (struct session *sess, char *mask, char *who, char *when, int is_exempt)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
 
-	store = get_store (sess);
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, mask, 1, who, 2, when, -1);
-
+	if (is_exempt)
+	{
+	} else
+	{
+		store = get_store (sess);
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, mask, 1, who, 2, when, -1);
+	}
 }
 
 void
-fe_ban_list_end (struct session *sess)
+fe_ban_list_end (struct session *sess, int is_exemption)
 {
 	gtk_widget_set_sensitive (sess->res->banlist_butRefresh, TRUE);
 }
@@ -165,7 +170,7 @@ banlist_unban (GtkWidget * wid, struct session *sess)
 	while (gtk_tree_model_iter_next (model, &iter));
 
 	/* and send to server */
-	send_channel_modes (sess, tbuf, masks, 0, i, '-', 'b');
+	send_channel_modes (sess, tbuf, masks, 0, i, '-', 'b', 0);
 
 	/* now free everything, and refresh banlist */	
 	for (i=0; i < num_sel; i++)
