@@ -56,6 +56,9 @@
 #include <gtk/gtkinvisible.h>
 #endif
 
+#ifdef WIN32
+#include <windows.h>
+#endif
 
 GdkPixmap *channelwin_pix;
 
@@ -130,22 +133,47 @@ fe_args (int argc, char *argv[])
 			printf (PACKAGE" "VERSION"\n");
 			return 0;
 		}
+		if (!strcasecmp (argv[1], "-p"))
+		{
+#ifdef WIN32
+			/* see the chdir() below */
+			char *sl, *exe = strdup (argv[0]);
+			sl = strrchr (exe, '\\');
+			if (sl)
+			{
+				*sl = 0;
+				printf ("%s\\plugins\n", exe);
+			}
+#else
+			printf ("%s\n", XCHATLIBDIR"/plugins");
+#endif
+			return 0;
+		}
+		if (!strcasecmp (argv[1], "-u"))
+		{
+			printf ("%s\n", get_xdir_fs ());
+			return 0;
+		}
 		if (!strcasecmp (argv[1], "-h") || !strcasecmp (argv[1], "--help"))
 		{
 			printf (PACKAGE" "VERSION"\n"
 					"Usage: %s [OPTIONS]... [URL]\n\n", argv[0]);
 			printf ("%s:\n"
-					"  -d,  --cfgdir %-11s %s\n"
 					"  -a,  --no-auto            %s\n"
+					"  -d,  --cfgdir %-11s %s\n"
 					"  -n,  --no-plugins         %s\n"
+					"  -p                        %s\n"
+					"  -u                        %s\n"
 					"  -v,  --version            %s\n\n"
 					"URL:\n"
 					"  irc://server:port/channel\n\n",
 						_("Options"),
+						_("don't auto connect"),
 						_("DIRECTORY"),
 						_("use a different config dir"),
-						_("don't auto connect"),
 						_("don't auto load any plugins"),
+						_("show plugin auto-load dir"),
+						_("show user config dir"),
 						_("show version information")
 					);
 			return 0;
@@ -240,14 +268,14 @@ create_input_style (void)
 	if (prefs.style_inputbox && !done_rc)
 	{
 		done_rc = TRUE;
-		sprintf (buf, cursor_color_rc, (colors[18].red >> 8),
-			(colors[18].green >> 8), (colors[18].blue >> 8));
+		sprintf (buf, cursor_color_rc, (colors[COL_FG].red >> 8),
+			(colors[COL_FG].green >> 8), (colors[COL_FG].blue >> 8));
 		gtk_rc_parse_string (buf);
 	}
 
-	style->bg[GTK_STATE_NORMAL] = colors[18];
-	style->base[GTK_STATE_NORMAL] = colors[19];
-	style->text[GTK_STATE_NORMAL] = colors[18];
+	style->bg[GTK_STATE_NORMAL] = colors[COL_FG];
+	style->base[GTK_STATE_NORMAL] = colors[COL_BG];
+	style->text[GTK_STATE_NORMAL] = colors[COL_FG];
 
 	return style;
 }
@@ -277,7 +305,8 @@ fe_main (void)
 void
 fe_cleanup (void)
 {
-	palette_save ();
+	/* it's saved when pressing OK in setup.c */
+	/*palette_save ();*/
 }
 
 void
@@ -313,7 +342,7 @@ log_handler (const gchar   *log_domain,
 
 	sess = find_dialog (serv_list->data, "(warnings)");
 	if (!sess)
-		sess = new_ircwindow (serv_list->data, "(warnings)", SESS_DIALOG);
+		sess = new_ircwindow (serv_list->data, "(warnings)", SESS_DIALOG, 0);
 
 	PrintTextf (sess, "%s\t%s\n", log_domain, message);
 	if (getenv ("XCHAT_WARNING_ABORT"))
@@ -323,7 +352,7 @@ log_handler (const gchar   *log_domain,
 #endif
 
 void
-fe_new_window (session *sess)
+fe_new_window (session *sess, int focus)
 {
 	int tab = FALSE;
 
@@ -337,7 +366,7 @@ fe_new_window (session *sess)
 			tab = TRUE;
 	}
 
-	mg_changui_new (sess, NULL, tab);
+	mg_changui_new (sess, NULL, tab, focus);
 
 #ifdef WIN32
 	g_log_set_handler ("GLib", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
@@ -435,6 +464,8 @@ fe_set_hilight (struct session *sess)
 {
 	if (sess->gui->is_tab)
 		fe_set_tab_color (sess, 3, TRUE);	/* blue, with taskbar flash */
+	else
+		fe_set_tab_color (sess, -1, TRUE);	/* taskbar flash only */
 }
 
 static void
@@ -706,6 +737,9 @@ fe_ctrl_gui (session *sess, int action, int arg)
 		fe_set_tab_color (sess, arg, FALSE); break;
 	case 5:
 		gtk_window_iconify (GTK_WINDOW (sess->gui->window)); break;
+	case 6:
+		menu_bar_toggle ();	/* toggle menubar on/off */
+		break;
 	}
 }
 
@@ -736,3 +770,105 @@ fe_gui_info (session *sess, int info_type)
 
 	return -1;
 }
+
+char *
+fe_get_inputbox_contents (session *sess)
+{
+	/* not the current tab */
+	if (sess->res->input_text)
+		return sess->res->input_text;
+
+	/* current focused tab */
+	return GTK_ENTRY (sess->gui->input_box)->text;
+}
+
+int
+fe_get_inputbox_cursor (session *sess)
+{
+	/* not the current tab (we don't remember the cursor pos) */
+	if (sess->res->input_text)
+		return 0;
+
+	/* current focused tab */
+	return gtk_editable_get_position (GTK_EDITABLE (sess->gui->input_box));
+}
+
+void
+fe_set_inputbox_cursor (session *sess, int delta, int pos)
+{
+	if (!sess->gui->is_tab || sess == current_tab)
+	{
+		if (delta)
+			pos += gtk_editable_get_position (GTK_EDITABLE (sess->gui->input_box));
+		gtk_editable_set_position (GTK_EDITABLE (sess->gui->input_box), pos);
+	} else
+	{
+		/* we don't support changing non-front tabs yet */
+	}
+}
+
+void
+fe_set_inputbox_contents (session *sess, char *text)
+{
+	if (!sess->gui->is_tab || sess == current_tab)
+	{
+		gtk_entry_set_text (GTK_ENTRY (sess->gui->input_box), text);
+	} else
+	{
+		if (sess->res->topic_text)
+			free (sess->res->topic_text);
+		sess->res->topic_text = strdup (text);
+	}
+}
+
+static void
+fe_open_url_locale (const char *url)
+{
+#ifdef WIN32
+	ShellExecute (0, "open", url, NULL, NULL, SW_SHOWNORMAL);
+#else
+	char tbuf[512], *moz;
+
+	/* gnome 2.4+ has this */
+	moz = g_find_program_in_path ("gnome-open");
+	if (moz)
+	{
+		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
+		g_free (moz);
+		xchat_exec (tbuf);
+		return;
+	}
+
+	moz = g_find_program_in_path ("gnome-moz-remote");
+	if (moz)
+	{
+		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
+		g_free (moz);
+	} else
+	{
+		snprintf (tbuf, sizeof (tbuf), "mozilla -remote 'openURL(%s)'", url);
+	}
+	xchat_exec (tbuf);
+#endif
+}
+
+void
+fe_open_url (const char *url)
+{
+	char *loc;
+
+	if (prefs.utf8_locale)
+	{
+		fe_open_url_locale (url);
+		return;
+	}
+
+	/* the OS expects it in "locale" encoding. This makes it work on
+	   unix systems that use ISO-8859-x and Win32. */
+	loc = g_locale_from_utf8 (url, -1, 0, 0, 0);
+	if (loc)
+	{
+		fe_open_url_locale (loc);
+		g_free (loc);
+	}
+}
