@@ -264,6 +264,13 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 		return FALSE;
 	current_sess = sess;
 
+	if (plugin_emit_keypress (sess, evt->state, evt->keyval, evt->length, evt->string))
+		return 1;
+
+	/* maybe the plugin closed this tab? */
+	if (!is_session (sess))
+		return 1;
+
 	mod = evt->state & (STATE_CTRL | STATE_ALT | STATE_SHIFT);
 
 	kb = keys_root;
@@ -402,7 +409,8 @@ key_load_defaults ()
 		"A\nLeft\nMove front tab left\nD1!\nD2!\n\n"\
 		"A\nRight\nMove front tab right\nD1!\nD2!\n\n"\
 		"CS\nPrior\nMove tab family left\nD1!\nD2!\n\n"\
-		"CS\nNext\nMove tab family right\nD1!\nD2!\n\n"
+		"CS\nNext\nMove tab family right\nD1!\nD2!\n\n"\
+		"None\nF9\nRun Command\nD1:/GUI MENU TOGGLE\nD2!\n\n"
 	char buf[512];
 	int fd;
 
@@ -736,7 +744,7 @@ key_dialog_show ()
 
 	vbox2 = gtk_vbox_new (0, 2);
 	gtk_box_pack_end (GTK_BOX (hbox), vbox2, 1, 1, 0);
-	wid = gtk_button_new_with_label (_("Add new"));
+	wid = gtk_button_new_with_label (_("Add New"));
 	gtk_box_pack_start (GTK_BOX (vbox2), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (key_dialog_add_new), list);
