@@ -44,12 +44,12 @@
 #include "../common/xchat.h"
 #include "../common/fe.h"
 #include "../common/server.h"
-#include "../common/text.h"
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
 #include "../common/inbound.h"
 #include "../common/plugin.h"
 #include "../common/modes.h"
+#include "../common/url.h"
 #include "fe-gtk.h"
 #include "banlist.h"
 #include "gtkutil.h"
@@ -164,6 +164,38 @@ flash_window (GtkWidget *win)
 	flash (&fi);
 	/*FlashWindowEx (&fi);*/
 }
+#else
+
+#ifdef USE_XLIB
+#include <gdk/gdkx.h>
+
+static void
+set_window_urgency (GtkWidget *win, gboolean set)
+{
+	XWMHints *hints;
+
+	hints = XGetWMHints(GDK_WINDOW_XDISPLAY(win->window), GDK_WINDOW_XWINDOW(win->window));
+	if (set)
+		hints->flags |= XUrgencyHint;
+	else
+		hints->flags &= ~XUrgencyHint;
+	XSetWMHints(GDK_WINDOW_XDISPLAY(win->window),
+	            GDK_WINDOW_XWINDOW(win->window), hints);
+	XFree(hints);
+}
+
+static void
+flash_window (GtkWidget *win)
+{
+	set_window_urgency (win, TRUE);
+}
+
+static void
+unflash_window (GtkWidget *win)
+{
+	set_window_urgency (win, FALSE);
+}
+#endif
 #endif
 
 /* set a tab plain, red, light-red, or blue */
@@ -202,8 +234,8 @@ fe_set_tab_color (struct session *sess, int col, int flash)
 		}
 	}
 
-#ifdef WIN32
-	if (flash && prefs.flash_hilight)
+#if defined(WIN32) || defined(USE_XLIB)
+	if (flash && prefs.flash_hilight && fe_gui_info (sess, 0) != 1)
 		flash_window (sess->gui->window);
 #endif
 }
@@ -582,7 +614,7 @@ mg_set_topic_tip (session *sess)
 			 GTK_ENTRY (sess->gui->topic_entry)->text[0])
 			add_tip (sess->gui->topic_entry, GTK_ENTRY (sess->gui->topic_entry)->text);
 		else
-			add_tip (sess->gui->topic_entry, ""); /* hSP: set topic to empty in case it isn't set.. */
+			add_tip (sess->gui->topic_entry, NULL);
 	}
 }
 
@@ -794,6 +826,8 @@ mg_topdestroy_cb (GtkWidget *win, session *sess)
 {
 /*	printf("enter mg_topdestroy. sess %p was destroyed\n", sess);*/
 
+	/* kill the text buffer */
+	gtk_xtext_buffer_free (sess->res->buffer);
 	/* kill the user list */
 	g_object_unref (G_OBJECT (sess->res->user_model));
 
@@ -844,14 +878,14 @@ mg_ircdestroy (GtkWidget *tab, session *sess)
 static void
 mg_tabdestroy_cb (GtkWidget *tab, session *sess)
 {
-	if (tab == active_tab && !xchat_is_quitting)
+	if (tab == active_tab && mg_gui)
 		mg_find_replacement_focus (tab);
 
 	if (sess == NULL)
 		mg_gendestroy (tab);
 	else
 	{
-		if (!xchat_is_quitting)
+		if (mg_gui)	/* NULL if the last tab was just killed via WM */
 			tab_group_cleanup (sess->gui->tabs_box);
 		mg_ircdestroy (tab, sess);
 	}
@@ -910,7 +944,7 @@ mg_create_sess_tree (GtkWidget *menu)
 		if (serv->servername[0] == 0)
 			top_item = gtk_menu_item_new_with_label (_("<none>"));
 		else
-			top_item = gtk_menu_item_new_with_label (get_network (serv->front_session, TRUE));
+			top_item = gtk_menu_item_new_with_label (server_get_network (serv, TRUE));
 
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), top_item);
 		gtk_widget_show (top_item);
@@ -942,9 +976,10 @@ mg_create_sess_tree (GtkWidget *menu)
 }
 
 static void
-mg_menu_destroy (GtkMenuShell *menushell, GtkWidget *menu)
+mg_menu_destroy (GtkWidget *menu, gpointer userdata)
 {
 	gtk_widget_destroy (menu);
+	g_object_unref (menu);
 }
 
 static void
@@ -1084,9 +1119,6 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 		gtk_widget_show (item);
 	}
 
-	if (sess)
-		mg_create_color_menu (menu, sess);
-
 	item = gtk_menu_item_new_with_label (_("Go to"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -1109,18 +1141,71 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 	if (sess && tabmenu_list)
 		menu_create (menu, tabmenu_list, sess->channel, FALSE);
 
+	if (event->window)
+		gtk_menu_set_screen (GTK_MENU (menu), gdk_drawable_get_screen (event->window));
+	g_object_ref (menu);
+	gtk_object_sink (GTK_OBJECT (menu));
 	g_signal_connect (G_OBJECT (menu), "selection-done",
-							G_CALLBACK (mg_menu_destroy), menu);
+							G_CALLBACK (mg_menu_destroy), NULL);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
 	return TRUE;
 }
 
+void
+mg_dnd_drop_file (session *sess, char *target, char *uri)
+{
+	char *p, *data, *next, *fname;
+
+	p = data = strdup (uri);
+	while (*p)
+	{
+		next = strchr (p, '\r');
+		if (strncasecmp ("file:", p, 5) == 0)
+		{
+			if (next)
+				*next = 0;
+			fname = g_filename_from_uri (p, NULL, NULL);
+			if (fname)
+			{
+				/* dcc_send() expects utf-8 */
+				p = g_filename_to_utf8 (fname, -1, 0, 0, 0);
+				if (p)
+				{
+					dcc_send (sess, target, p, prefs.dcc_max_send_cps, 0);
+					g_free (p);
+				}
+				g_free (fname);
+			}
+		}
+		if (!next)
+			break;
+		p = next + 1;
+		if (*p == '\n')
+			p++;
+	}
+	free (data);
+
+}
+
+static void
+mg_dialog_dnd_drop (GtkWidget * widget, GdkDragContext * context, gint x,
+						  gint y, GtkSelectionData * selection_data, guint info,
+						  guint32 time, session *sess)
+{
+	/* sess->channel is really the nickname of dialogs */
+	mg_dnd_drop_file (sess, sess->channel, selection_data->data);
+}
+
 /* add a tabbed channel */
 
 static void
 mg_add_chan (session *sess)
 {
 	char *name = _("<none>");
+	static const GtkTargetEntry dnd_targets[] =
+	{
+		{"text/uri-list", 0, 1}
+	};
 
 	if (sess->channel[0])
 		name = sess->channel;
@@ -1143,6 +1228,16 @@ mg_add_chan (session *sess)
 		gtk_xtext_set_time_stamp (sess->res->buffer, prefs.timestamp);
 		sess->res->user_model = userlist_create_model ();
 	}
+
+#ifndef WIN32	/* needs more work */
+	if (sess->type == SESS_DIALOG)
+	{
+		gtk_drag_dest_set (sess->gui->xtext, GTK_DEST_DEFAULT_ALL, dnd_targets,
+								 1, GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
+		g_signal_connect (G_OBJECT (sess->gui->xtext), "drag_data_received",
+								G_CALLBACK (mg_dialog_dnd_drop), sess);
+	}
+#endif
 }
 
 static void
@@ -1277,7 +1372,8 @@ mg_dcc_active (void)
 	while (list)
 	{
 		dcc = list->data;
-		if (dcc->dccstat == STAT_ACTIVE)
+		if ((dcc->type == TYPE_SEND || dcc->type == TYPE_RECV) &&
+			 dcc->dccstat == STAT_ACTIVE)
 			return 1;
 		list = list->next;
 	}
@@ -1372,6 +1468,8 @@ mg_link_irctab (session *sess, int focus)
 	mg_unpopulate (sess);
 	mg_changui_destroy (sess);
 	mg_changui_new (sess, sess->res, 1, focus);
+	/* the buffer is now attached to a different widget */
+	((xtext_buffer *)sess->res->buffer)->xtext = (GtkXText *)sess->gui->xtext;
 }
 
 static void
@@ -1712,18 +1810,30 @@ mg_create_topicbar (session *sess, GtkWidget *box, char *name)
 /* check if a word is clickable */
 
 static int
-mg_word_check (GtkWidget * xtext, char *word)
+mg_word_check (GtkWidget * xtext, char *word, int len)
 {
-	return text_word_check (word);	/* common/text.c */
+	session *sess = current_sess;
+	int ret;
+
+	ret = url_check_word (word, len);	/* common/url.c */
+	if (ret == 0)
+	{
+		if (( (word[0]=='@' || word[0]=='+') && find_name (sess, word+1)) || find_name (sess, word))
+			return WORD_NICK;
+
+		if (sess->type == SESS_DIALOG)
+			return WORD_DIALOG;
+	}
+
+	return ret;
 }
 
 /* mouse click inside text area */
 
 static void
-mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
-					  session *sess)
+mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even)
 {
-	sess = current_sess;
+	session *sess = current_sess;
 
 	if (even->button == 1)			/* left button */
 	{
@@ -1735,7 +1845,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 
 		if (even->state & GDK_CONTROL_MASK)
 		{
-			switch (mg_word_check (xtext, word))
+			switch (mg_word_check (xtext, word, strlen (word)))
 			{
 			case WORD_URL:
 			case WORD_HOST:
@@ -1754,7 +1864,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 		return;
 	}
 
-	switch (mg_word_check (xtext, word))
+	switch (mg_word_check (xtext, word, strlen (word)))
 	{
 	case 0:
 		menu_middlemenu (sess, even);
@@ -2074,6 +2184,12 @@ mg_set_tabs_pos (session_gui *gui, int pos)
 		gtk_widget_unref (gui->tabs_box);
 }
 
+static void
+mg_inputbox_rightclick (GtkEntry *entry, GtkWidget *menu)
+{
+	mg_create_color_menu (menu, NULL);
+}
+
 static void
 mg_create_entry (session *sess, GtkWidget *box)
 {
@@ -2102,6 +2218,8 @@ mg_create_entry (session *sess, GtkWidget *box)
 							G_CALLBACK (mg_inputbox_focus), gui);
 	g_signal_connect (G_OBJECT (entry), "activate",
 							G_CALLBACK (mg_inputbox_cb), gui);
+	g_signal_connect (G_OBJECT (entry), "populate_popup",
+							G_CALLBACK (mg_inputbox_rightclick), NULL);
 	gtk_container_add (GTK_CONTAINER (hbox), entry);
 	gtk_widget_grab_focus (entry);
 
@@ -2177,6 +2295,11 @@ mg_tabwin_focus_cb (GtkWindow * win, GdkEventFocus *event, gpointer userdata)
 		gtk_xtext_check_marker_visibility (GTK_XTEXT (current_sess->gui->xtext));
 		plugin_emit_dummy_print (current_sess, "Focus Window");
 	}
+#ifndef WIN32
+#ifdef USE_XLIB
+	unflash_window (GTK_WIDGET (win));
+#endif
+#endif
 	return FALSE;
 }
 
@@ -2187,6 +2310,11 @@ mg_topwin_focus_cb (GtkWindow * win, GdkEventFocus *event, session *sess)
 	if (!sess->server->server_session)
 		sess->server->server_session = sess;
 	gtk_xtext_check_marker_visibility(GTK_XTEXT (current_sess->gui->xtext));
+#ifndef WIN32
+#ifdef USE_XLIB
+	unflash_window (GTK_WIDGET (win));
+#endif
+#endif
 	plugin_emit_dummy_print (sess, "Focus Window");
 	return FALSE;
 }
@@ -2797,7 +2925,7 @@ fe_session_callback (session *sess)
 	if (sess->gui->bartag)
 		fe_timeout_remove (sess->gui->bartag);
 
-	if (sess->gui != mg_gui)
+	if (sess->gui != &static_mg_gui)
 		free (sess->gui);
 	free (sess->res);
 }
