@@ -32,6 +32,7 @@ static int tab_right_is_moving = 0;
  *
  * group:
  *   "c" tab-focus callback function
+ *   "o" compare function
  *   "foc" currently focused tab
  *   "i" inner hbox (inside the viewport)
  *   "s" sorted (boolean)
@@ -265,8 +266,20 @@ tab_pressed_cb (GtkToggleButton *tab, GtkWidget *group)
 	}
 }
 
+static gboolean
+tab_scroll_cb (GtkWidget *widget, GdkEventScroll *event, gpointer group)
+{
+	/* mouse wheel scrolling */
+	if (event->direction == GDK_SCROLL_UP)
+		tab_scroll_left_up_clicked (widget, group);
+	else if (event->direction == GDK_SCROLL_DOWN)
+		tab_scroll_right_down_clicked (widget, group);
+
+	return FALSE;
+}
+
 GtkWidget *
-tab_group_new (void *callback, gboolean vertical, gboolean sorted)
+tab_group_new (void *callback, void *compare, gboolean vertical, gboolean sorted)
 {
 	GtkWidget *box;
 	GtkWidget *viewport;
@@ -281,6 +294,7 @@ tab_group_new (void *callback, gboolean vertical, gboolean sorted)
 	} else
 		group = gtk_hbox_new (0, 0);
 	g_object_set_data (G_OBJECT (group), "c", callback);
+	g_object_set_data (G_OBJECT (group), "o", compare);
 	g_signal_connect (G_OBJECT (group), "size_allocate",
 							G_CALLBACK (tab_group_resize), NULL);
 	gtk_container_set_border_width (GTK_CONTAINER (group), 2);
@@ -311,6 +325,8 @@ tab_group_new (void *callback, gboolean vertical, gboolean sorted)
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT (button), "clicked",
 							G_CALLBACK (tab_scroll_right_down_clicked), group);
+	g_signal_connect (G_OBJECT (button), "scroll_event",
+							G_CALLBACK (tab_scroll_cb), group);
 	gtk_box_pack_end (GTK_BOX (group), button, 0, 0, 0);
 	gtk_widget_show (arrow);
 
@@ -322,6 +338,8 @@ tab_group_new (void *callback, gboolean vertical, gboolean sorted)
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT (button), "clicked",
 							G_CALLBACK (tab_scroll_left_up_clicked), group);
+	g_signal_connect (G_OBJECT (button), "scroll_event",
+							G_CALLBACK (tab_scroll_cb), group);
 	gtk_box_pack_end (GTK_BOX (group), button, 0, 0, 0);
 	gtk_widget_show (arrow);
 
@@ -521,8 +539,9 @@ tab_add_sorted (GtkWidget *group, GtkWidget *box, GtkWidget *tab)
 {
 	GList *list;
 	GtkBoxChild *child;
-	char *name;
 	int i = 0;
+	int (*cmp) (void *a, void *b);
+	void *b;
 
 	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (group), "s")))
 	{
@@ -532,26 +551,23 @@ tab_add_sorted (GtkWidget *group, GtkWidget *box, GtkWidget *tab)
 	}
 
 	/* sorting TODO:
-    *   - always put servertab first (dialogs get mixed in)
-    *   - move tab if renamed */
-	name = GTK_BUTTON (tab)->label_text;
+    *   - move tab if renamed (dialogs) */
+
+	/* userdata, passed to mg_tabs_compare() */
+	b = g_object_get_data (G_OBJECT (tab), "u");
+
+	/* compare function, as set by tab_group_new() */
+	cmp = g_object_get_data (G_OBJECT (group), "o");
+
 	list = GTK_BOX (box)->children;
 	while (list)
 	{
 		child = list->data;
 		if (!GTK_IS_SEPARATOR (child->widget))
 		{
-			gboolean insert_now = FALSE;
-			char *label_text = GTK_BUTTON (child->widget)->label_text;
+			void *a = g_object_get_data (G_OBJECT (child->widget), "u");
 
-			if (strcasecmp (label_text, name) > 0)
-				insert_now = TRUE;
-
-			/* channels get a lower priority, and are placed last */
-			if (name[0] == '#' && label_text[0] != '#')
-				insert_now = FALSE;
-
-			if (insert_now)
+			if (cmp (a, b) > 0)
 			{
 				gtk_box_pack_start (GTK_BOX (box), tab, 0, 0, 0);
 				gtk_box_reorder_child (GTK_BOX (box), tab, i);
@@ -772,7 +788,10 @@ tab_rename (GtkWidget *tab, char *name, int trunc_len)
 
 	/* Toplevel window's tabs are not working here, why? */
 	if (GTK_IS_LABEL (GTK_BIN (tab)->child))
+	{
 		attr = gtk_label_get_attributes (GTK_LABEL (GTK_BIN (tab)->child));
+		pango_attr_list_ref (attr);
+	}
 
 	new_name = truncate_tab_name (name, trunc_len);
 	if (new_name)
@@ -784,7 +803,10 @@ tab_rename (GtkWidget *tab, char *name, int trunc_len)
 	gtk_widget_queue_resize (tab->parent->parent->parent);
 
 	if (attr)
+	{
 		gtk_label_set_attributes (GTK_LABEL (GTK_BIN (tab)->child), attr);
+		pango_attr_list_unref (attr);
+	}
 }
 
 void
@@ -881,6 +903,7 @@ tab_group_set_orientation (GtkWidget *group, gboolean vertical)
 		return group;
 
 	new_group = tab_group_new (g_object_get_data (G_OBJECT (group), "c"),
+										g_object_get_data (G_OBJECT (group), "o"),
 										vertical,
 						/* sort: boolean */
 						GPOINTER_TO_INT (g_object_get_data (G_OBJECT (group), "s")));
