@@ -36,6 +36,7 @@
 
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
+#include "../common/fe.h"
 #include "../common/url.h"
 #include "../common/tree.h"
 #include "gtkutil.h"
@@ -144,12 +145,12 @@ url_save_callback (void *arg1, char *file)
 static void
 url_button_save (void)
 {
-	gtkutil_file_req (_("Select a file to save to"),
+	gtkutil_file_req (_("Select an output filename"),
 							url_save_callback, NULL, NULL, FRF_WRITE);
 }
 
 void
-fe_url_add (char *urltext)
+fe_url_add (const char *urltext)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
