@@ -82,7 +82,7 @@ read_stdin (void)
 static int done_intro = 0;
 
 void
-fe_new_window (struct session *sess)
+fe_new_window (struct session *sess, int focus)
 {
 	char buf[512];
 
@@ -99,12 +99,12 @@ fe_new_window (struct session *sess)
 		return;
 	done_intro = 1;
 
-	snprintf (buf, sizeof (buf),	
+	snprintf (buf, sizeof (buf),
 				"\n"
 				" \017xchat \00310"VERSION"\n"
 				" \017Running on \00310%s \017glib \00310%d.%d.%d\n"
 				" \017This binary compiled \00310"__DATE__"\017\n",
-				get_cpu_str(), 
+				get_cpu_str(),
 				glib_major_version, glib_minor_version, glib_micro_version);
 	fe_print_text (sess, buf);
 
@@ -121,21 +121,12 @@ fe_new_window (struct session *sess)
 #ifdef ENABLE_NLS
 	"NLS "
 #endif
-#ifdef USE_TRANS
-	"Trans "
-#endif
-#ifdef USE_HEBREW
-	"Hebrew "
-#endif
 #ifdef USE_OPENSSL
 	"OpenSSL "
 #endif
 #ifdef SOCKS
 	"Socks5 "
 #endif
-#ifdef USE_JCODE
-	"JCode "
-#endif
 #ifdef USE_IPV6
 	"IPv6"
 #endif
@@ -443,7 +434,7 @@ fe_main (void)
 	guint64 shortest, delay;
 
 	if (!sess_list)
-		new_ircwindow (NULL, NULL, SESS_SERVER);
+		new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 
 #ifdef ENABLE_NLS
 	bindtextdomain (PACKAGE, PREFIX"/share/locale");
@@ -636,11 +627,11 @@ fe_is_banwindow (struct session *sess)
 	return 0;
 }
 void
-fe_add_ban_list (struct session *sess, char *chan, char *users, char *topic)
+fe_add_ban_list (struct session *sess, char *mask, char *who, char *when, int is_exemption)
 {
-}               
+}
 void
-fe_ban_list_end (struct session *sess)
+fe_ban_list_end (struct session *sess, int is_exemption)
 {
 }
 void
@@ -815,3 +806,32 @@ fe_gui_info (session *sess, int info_type)
 void fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud)
 {
 }
+char *fe_get_inputbox_contents (struct session *sess)
+{
+	return NULL;
+}
+void fe_set_inputbox_contents (struct session *sess, char *text)
+{
+}
+int fe_get_inputbox_cursor (struct session *sess)
+{
+	return 0;
+}
+void fe_set_inputbox_cursor (struct session *sess, int delta, int pos)
+{
+}
+void fe_open_url (const char *url)
+{
+}
+void fe_menu_del (menu_entry *me)
+{
+}
+void fe_menu_add (menu_entry *me)
+{
+}
+void fe_menu_update (menu_entry *me)
+{
+}
+void fe_uselect (struct session *sess, char *word[], int do_clear, int scroll_to)
+{
+}
